@@ -1,4 +1,4 @@
-/* $Id: gdict-applet.c,v 1.58 2004/05/08 01:59:49 gman Exp $ */
+/* $Id: gdict-applet.c,v 1.62 2004/10/19 16:38:45 vnoel Exp $ */
 
 /*
  *  Papadimitriou Spiros <spapadim+@cs.cmu.edu>
@@ -107,6 +107,30 @@ lookup_button_drag_cb (GtkWidget *widget, GdkDragContext *context, gint x, gint
 	
 }
 
+/* Force the button not to have any focus padding and let the focus
+ * indication be drawn on the label itself when space is tight. Taken from the clock applet.
+ * FIXME : This is a hack, a more elegant solution should be found, maybe at the gtk+ level. */
+
+static inline void force_no_focus_padding (GtkWidget *widget)
+{
+  gboolean first_time=TRUE;
+
+  if (first_time) {
+    gtk_rc_parse_string ("\n"
+			 "   style \"gdict-applet-button-style\"\n"
+			 "   {\n"
+			 "      GtkWidget::focus-line-width=0\n"
+			 "      GtkWidget::focus-padding=0\n"
+			 "   }\n"
+			 "\n"
+			 "    widget \"*.gdict-applet-button\" style \"gdict-applet-button-style\"\n"
+			 "\n");
+    first_time = FALSE;
+  }
+
+  gtk_widget_set_name (widget, "gdict-applet-button");
+}
+
 static void
 setup_applet_widgets (GDictApplet *applet)
 {
@@ -118,9 +142,6 @@ setup_applet_widgets (GDictApplet *applet)
   		{ "STRING",     0, 0 }
     	};
 	
-	gboolean use_horizontal = TRUE;
-
-
 	applet->about_dialog = NULL;
 
 	if (applet->box) {
@@ -128,20 +149,17 @@ setup_applet_widgets (GDictApplet *applet)
 	}
 
 	if ((applet->orient == PANEL_APPLET_ORIENT_LEFT || applet->orient == PANEL_APPLET_ORIENT_RIGHT) || 
-	    ((applet->orient == PANEL_APPLET_ORIENT_UP || applet->orient == PANEL_APPLET_ORIENT_DOWN) && applet->panel_size >= 48)) {
-		use_horizontal = FALSE;
-	} 
-	
-	if (use_horizontal == TRUE) {
-		applet->box = gtk_hbox_new (FALSE, 2);
-	}
-	else {
+	    ((applet->orient == PANEL_APPLET_ORIENT_UP || applet->orient == PANEL_APPLET_ORIENT_DOWN) && applet->panel_size > 48)) {
 		applet->box = gtk_vbox_new (FALSE, 2);
 	}
-
+	else {
+		applet->box = gtk_hbox_new (FALSE, 2);
+	} 
+	
 	gtk_container_set_border_width (GTK_CONTAINER (applet->box), 0);
 
 	applet->button_widget = gtk_button_new_with_mnemonic (_("_Lookup"));
+	force_no_focus_padding (applet->button_widget);
 
 	gtk_drag_dest_set (applet->button_widget, GTK_DEST_DEFAULT_ALL, drop_targets, 
     		           G_N_ELEMENTS (drop_targets), GDK_ACTION_COPY | GDK_ACTION_MOVE);
@@ -157,24 +175,15 @@ setup_applet_widgets (GDictApplet *applet)
 	gtk_entry_set_width_chars (GTK_ENTRY (applet->entry_widget), 12);
 	gtk_box_pack_start (GTK_BOX (applet->box), applet->entry_widget, FALSE, FALSE, 0);
 
-	if (use_horizontal == TRUE) {
-		gtk_widget_set_size_request (GTK_WIDGET (applet->entry_widget), -1, applet->panel_size + 4);
-	}
-	else {
-		if (applet->orient == PANEL_APPLET_ORIENT_UP || applet->orient == PANEL_APPLET_ORIENT_DOWN) {
-			gtk_widget_set_size_request (GTK_WIDGET (applet->entry_widget), -1, (applet->panel_size + 4) / 2);
-		}
-		else {
-			gtk_widget_set_size_request (GTK_WIDGET (applet->entry_widget), -1, -1);
-		}
-	}
- 	
  	gtk_widget_show_all (applet->box);
 	  	
  	gtk_container_add(GTK_CONTAINER(applet->applet_widget), applet->box);                              
 
  	gtk_widget_show (applet->applet_widget);
-	
+
+	gtk_tooltips_set_tip (GTK_TOOLTIPS(applet->tooltip_widget), applet->entry_widget,
+			      _("Lookup words in an online dictionary"), NULL);
+
  	g_signal_connect (G_OBJECT (applet->entry_widget), "activate",
  			  G_CALLBACK (entry_activate_cb), applet);
 			  
@@ -405,21 +414,18 @@ gdict_applet_new (PanelApplet *parent_applet)
  	
 	panel_applet_set_flags (parent_applet, PANEL_APPLET_EXPAND_MINOR);
 
+	applet->tooltip_widget = GTK_WIDGET(gtk_tooltips_new ());
+	g_object_ref (applet->tooltip_widget);
+	gtk_object_sink (GTK_OBJECT (applet->tooltip_widget));
+	g_object_set_data_full (G_OBJECT (applet->applet_widget), "tooltips",
+				applet->tooltip_widget, (GDestroyNotify) g_object_unref);
+
 	setup_applet_widgets (applet);
 
  	gdict_pref_load ();
  	
  	gdict_app_create (TRUE);
  	
-	tooltips = gtk_tooltips_new ();
-	g_object_ref (tooltips);
-	gtk_object_sink (GTK_OBJECT (tooltips));
-	g_object_set_data_full (G_OBJECT (applet->applet_widget), "tooltips",
-				tooltips, (GDestroyNotify) g_object_unref);
-
-	gtk_tooltips_set_tip (tooltips, applet->applet_widget,
-			      _("Lookup words in an online dictionary"), NULL);
-
  	panel_applet_setup_menu_from_file (PANEL_APPLET (applet->applet_widget),
 					   NULL,
                             		   "GNOME_GDictApplet.xml",
