@@ -1,7 +1,7 @@
 #ifndef __GDICT_APPLET_H_
 #define __GDICT_APPLET_H_
 
-/* $Id: gdict-applet.h,v 1.8 2003/06/05 06:10:42 dcransto Exp $ */
+/* $Id: gdict-applet.h,v 1.10 2004/07/19 02:30:42 gman Exp $ */
 
 /*
  *  Papadimitriou Spiros <spapadim@cs.cmu.edu>
@@ -21,10 +21,10 @@
 typedef struct _GDictApplet {
 	GtkWidget *applet_widget;
 	GtkWidget *button_widget;
-	GtkWidget *image;
 	GtkWidget *box;
 	GtkWidget *entry_widget;
-	GtkWidget *handlebox_widget;
+	GtkWidget *about_dialog;
+	GtkWidget *tooltip_widget;
 	PanelAppletOrient orient;
 	gboolean handle;
 	gint panel_size;
