@@ -32,8 +32,10 @@
 #include <gnome.h>
 
 #include <regex.h>
+#include <libgnomevfs/gnome-vfs-mime.h>
 #include <libgnomevfs/gnome-vfs-mime-handlers.h>
 #include <libgnomevfs/gnome-vfs-ops.h> 
+#include <libgnomevfs/gnome-vfs-utils.h>
 #include <bonobo-activation/bonobo-activation.h>
 
 #include "gsearchtool-support.h"
@@ -42,9 +44,17 @@
 
 #define C_STANDARD_STRFTIME_CHARACTERS "aAbBcdHIjmMpSUwWxXyYZ"
 #define C_STANDARD_NUMERIC_STRFTIME_CHARACTERS "dHIjmMSUwWyY"
+#define SUS_EXTENDED_STRFTIME_MODIFIERS "EO"
 #define BINARY_EXEC_MIME_TYPE      "application/x-executable-binary"
 #define ICON_THEME_EXECUTABLE_ICON "gnome-fs-executable"
 #define ICON_THEME_REGULAR_ICON    "gnome-fs-regular"
+#define ICON_THEME_CHAR_DEVICE     "gnome-fs-chardev"
+#define ICON_THEME_BLOCK_DEVICE    "gnome-fs-blockdev"
+#define ICON_THEME_SOCKET          "gnome-fs-socket"
+#define ICON_THEME_FIFO            "gnome-fs-fifo"
+#define MAX_SYMLINKS_FOLLOWED      32
+#define GSEARCH_DATE_FORMAT_LOCALE "locale"
+#define GSEARCH_DATE_FORMAT_ISO    "iso"
 
 
 /* START OF THE GCONF FUNCTIONS */
@@ -106,6 +116,27 @@ gsearchtool_gconf_get_string (const gchar *key)
 	return result;
 }
 
+GSList * 
+gsearchtool_gconf_get_list (const gchar *key)
+{
+	GSList *result;
+	GConfClient *client;
+	GError *error = NULL;
+	
+	g_return_val_if_fail (key != NULL, FALSE);
+	
+	client = gsearchtool_gconf_client_get_global ();
+	g_return_val_if_fail (client != NULL, FALSE);
+	
+	result = gconf_client_get_list (client, key, GCONF_VALUE_STRING, &error);
+	
+	if (gsearchtool_gconf_handle_error (&error)) {
+		result = NULL;
+	}
+	
+	return result;
+}
+
 gboolean 
 gsearchtool_gconf_get_boolean (const gchar *key)
 {
@@ -143,6 +174,98 @@ gsearchtool_gconf_set_boolean (const gchar *key, const gboolean flag)
 	gsearchtool_gconf_handle_error (&error);
 }
 
+/* START OF THE GNOME-VFS RELATED FUNCTIONS */
+
+GnomeVFSResult
+uri_resolve_all_symlinks_uri (GnomeVFSURI *uri,
+			      GnomeVFSURI **result_uri)
+{
+	GnomeVFSURI *new_uri, *resolved_uri;
+	GnomeVFSFileInfo *info;
+	GnomeVFSResult res;
+	char *p;
+	int n_followed_symlinks;
+
+	/* Ref the original uri so we don't lose it */
+	uri = gnome_vfs_uri_ref (uri);
+
+	*result_uri = NULL;
+
+	info = gnome_vfs_file_info_new ();
+
+	p = uri->text;
+	n_followed_symlinks = 0;
+	while (*p != 0) {
+		while (*p == GNOME_VFS_URI_PATH_CHR)
+			p++;
+		while (*p != 0 && *p != GNOME_VFS_URI_PATH_CHR)
+			p++;
+
+		new_uri = gnome_vfs_uri_dup (uri);
+		g_free (new_uri->text);
+		new_uri->text = g_strndup (uri->text, p - uri->text);
+		
+		gnome_vfs_file_info_clear (info);
+		res = gnome_vfs_get_file_info_uri (new_uri, info, GNOME_VFS_FILE_INFO_DEFAULT);
+		if (res != GNOME_VFS_OK) {
+			gnome_vfs_uri_unref (new_uri);
+			goto out;
+		}
+		if (info->type == GNOME_VFS_FILE_TYPE_SYMBOLIC_LINK &&
+		    info->valid_fields & GNOME_VFS_FILE_INFO_FIELDS_SYMLINK_NAME) {
+			n_followed_symlinks++;
+			if (n_followed_symlinks > MAX_SYMLINKS_FOLLOWED) {
+				res = GNOME_VFS_ERROR_TOO_MANY_LINKS;
+				gnome_vfs_uri_unref (new_uri);
+				goto out;
+			}
+			resolved_uri = gnome_vfs_uri_resolve_relative (new_uri,
+								       info->symlink_name);
+			if (*p != 0) {
+				gnome_vfs_uri_unref (uri);
+				uri = gnome_vfs_uri_append_path (resolved_uri, p);
+				gnome_vfs_uri_unref (resolved_uri);
+			} else {
+				gnome_vfs_uri_unref (uri);
+				uri = resolved_uri;
+			}
+
+			p = uri->text;
+		} 
+		gnome_vfs_uri_unref (new_uri);
+	}
+
+	res = GNOME_VFS_OK;
+	*result_uri = gnome_vfs_uri_dup (uri);
+ out:
+	gnome_vfs_file_info_unref (info);
+	gnome_vfs_uri_unref (uri);
+	return res;
+}
+
+GnomeVFSResult
+uri_resolve_all_symlinks (const char *text_uri,
+	                  char **resolved_text_uri)
+{
+	GnomeVFSURI *uri, *resolved_uri;
+	GnomeVFSResult res;
+
+	*resolved_text_uri = NULL;
+
+	uri = gnome_vfs_uri_new (text_uri);
+	if (uri == NULL || uri->text == NULL) {
+		return GNOME_VFS_ERROR_NOT_SUPPORTED;
+	}
+
+	res = uri_resolve_all_symlinks_uri (uri, &resolved_uri);
+
+	if (res == GNOME_VFS_OK) {
+		*resolved_text_uri = gnome_vfs_uri_to_string (resolved_uri, GNOME_VFS_URI_HIDE_NONE);
+		gnome_vfs_uri_unref (resolved_uri);
+	}
+	return res;
+}
+
 /* START OF GENERIC GNOME-SEARCH-TOOL FUNCTIONS */
 
 gboolean 
@@ -173,22 +296,152 @@ is_path_hidden (const gchar *path)
 	return results;
 }
 
-gboolean 
-is_path_in_home_folder (const gchar *path)
+gboolean  	
+is_quick_search_excluded_path (const gchar *path)
 {
-	return (g_strstr_len (path, strlen (g_get_home_dir ()), g_get_home_dir ()) != NULL);
+	GSList     *exclude_path_list;
+	GSList     *tmp_list;
+	gchar      *dir;
+	
+	dir = g_strdup (path);
+	
+	/* Remove trailing G_DIR_SEPARATOR. */
+	if ((strlen (dir) > 1) && (g_str_has_suffix (dir, G_DIR_SEPARATOR_S) == TRUE)) {
+		dir[strlen (dir) - 1] = '\0'; 
+	}
+	
+	/* Always exclude a path that is symbolic link. */
+	if (g_file_test (dir, G_FILE_TEST_IS_SYMLINK)) {
+		g_free (dir);
+		
+		return TRUE;
+	}
+	g_free (dir);
+	
+	/* Check path against the Quick-Search-Excluded-Paths list. */
+	exclude_path_list = gsearchtool_gconf_get_list ("/apps/gnome-search-tool/quick_search_excluded_paths");
+
+	for (tmp_list = exclude_path_list; tmp_list; tmp_list = tmp_list->next) {
+	
+		gchar *dir;
+		
+		dir = g_strdup (tmp_list->data);
+		
+		/* Skip empty or null values. */
+		if ((dir == NULL) || (strlen (dir) == 0)) {
+			continue;
+		}
+			
+		/* Wild-card comparisons. */		
+		if (g_strstr_len (dir, strlen (dir), "*") != NULL) { 
+		
+			if (g_pattern_match_simple (dir, path) == TRUE) {
+
+				g_slist_free (exclude_path_list);
+				g_free (dir);			
+
+				return TRUE;
+			}
+		} 
+		/* Non-wild-card comparisons. */
+		else {
+			/* Add a trailing G_DIR_SEPARATOR. */
+			if (g_str_has_suffix (dir, G_DIR_SEPARATOR_S) == FALSE) {
+			 
+				gchar *tmp;
+			
+				tmp = dir;
+				dir = g_strconcat (dir, G_DIR_SEPARATOR_S, NULL);
+				g_free (tmp);
+			}
+			
+			if (strcmp (path, dir) == 0) {
+				
+				g_slist_free (exclude_path_list);
+				g_free (dir);
+				
+				return TRUE;
+			}
+		}
+		g_free (dir);
+	}
+	g_slist_free (exclude_path_list);
+	
+	return FALSE;
 }
 
-gboolean 
-is_path_in_mount_folder (const gchar *path)
+gboolean  	
+is_second_scan_excluded_path (const gchar *path)
 {
-	return (g_strstr_len (path, strlen ("/mnt/"), "/mnt/") != NULL);
-}
+	GSList     *exclude_path_list;
+	GSList     *tmp_list;
+	gchar      *dir;
+	
+	dir = g_strdup (path);
+	
+	/* Remove trailing G_DIR_SEPARATOR. */
+	if ((strlen (dir) > 1) && (g_str_has_suffix (dir, G_DIR_SEPARATOR_S) == TRUE)) {
+		dir[strlen (dir) - 1] = '\0'; 
+	}
+	
+	/* Always exclude a path that is symbolic link. */
+	if (g_file_test (dir, G_FILE_TEST_IS_SYMLINK)) {
+		g_free (dir);
+		
+		return TRUE;
+	}
+	g_free (dir);
+	
+	/* Check path against the Quick-Search-Excluded-Paths list. */
+	exclude_path_list = gsearchtool_gconf_get_list ("/apps/gnome-search-tool/quick_search_second_scan_excluded_paths");
 
-gboolean 
-is_path_in_tmp_folder (const gchar *path)
-{
-	return (g_strstr_len (path, strlen ("/tmp/"), "/tmp/") != NULL);
+	for (tmp_list = exclude_path_list; tmp_list; tmp_list = tmp_list->next) {
+	
+		gchar *dir;
+		
+		dir = g_strdup (tmp_list->data);
+		
+		/* Skip empty or null values. */
+		if ((dir == NULL) || (strlen (dir) == 0)) {
+			continue;
+		}
+			
+		/* Wild-card comparisons. */		
+		if (g_strstr_len (dir, strlen (dir), "*") != NULL) { 
+		
+			if (g_pattern_match_simple (dir, path) == TRUE) {
+
+				g_slist_free (exclude_path_list);
+				g_free (dir);			
+
+				return TRUE;
+			}
+		} 
+		/* Non-wild-card comparisons. */
+		else {
+			/* Add a trailing G_DIR_SEPARATOR. */
+			if (g_str_has_suffix (dir, G_DIR_SEPARATOR_S) == FALSE) {
+			 
+				gchar *tmp;
+			
+				tmp = dir;
+				dir = g_strconcat (dir, G_DIR_SEPARATOR_S, NULL);
+				g_free (tmp);
+			}
+			
+			if (strcmp (path, dir) == 0) {
+				
+				g_slist_free (exclude_path_list);
+				g_free (dir);
+				
+				return TRUE;
+			}
+		}
+		g_free (dir);
+	}
+	g_slist_free (exclude_path_list);
+	
+	return FALSE;
 }
 
 gboolean
@@ -211,6 +464,10 @@ compare_regex (const gchar *regex,
 {
 	regex_t regexec_pattern;
 	
+	if (regex == NULL) {
+		return TRUE;
+	}
+	
 	regcomp (&regexec_pattern, regex, REG_NOSUB);
 	
 	if (regexec (&regexec_pattern, string, 0, 0, 0) != REG_NOMATCH) {
@@ -261,6 +518,34 @@ escape_single_quotes (const gchar *string)
 	return g_string_free (gs, FALSE);
 }
 
+gchar *
+backslash_special_characters (const gchar *string)
+{
+	GString *gs;
+
+	if (string == NULL) {
+		return NULL;
+	}
+
+	if ((count_of_char_in_string (string, '\\') == 0) &&
+	    (count_of_char_in_string (string, '-') == 0)) {
+		return g_strdup(string);
+	}
+	gs = g_string_new ("");
+	for(; *string; string++) {
+		if (*string == '\\') {
+			g_string_append(gs, "\\\\");
+		}
+		else if (*string == '-') {
+			g_string_append(gs, "\\-");
+		}
+		else {
+			g_string_append_c(gs, *string);
+		}
+	}
+	return g_string_free (gs, FALSE);
+}
+
 gchar *
 remove_mnemonic_character (const gchar *string)
 {
@@ -304,6 +589,16 @@ get_readable_date (const time_t file_time_raw)
 	gchar *readable_date;
 
 	file_time = localtime (&file_time_raw);
+
+	/* Base format of date column on nautilus date_format key */
+	if (search_command.date_format_pref != NULL) {
+		if (strcmp(search_command.date_format_pref, GSEARCH_DATE_FORMAT_LOCALE) == 0) {
+			return gsearchtool_strdup_strftime ("%c", file_time);
+		} else if (strcmp (search_command.date_format_pref, GSEARCH_DATE_FORMAT_ISO) == 0) {
+			return gsearchtool_strdup_strftime ("%Y-%m-%d %H:%M:%S", file_time);
+		}
+	}
+	
 	file_date = g_date_new_dmy (file_time->tm_mday,
 			       file_time->tm_mon + 1,
 			       file_time->tm_year + 1900);
@@ -326,35 +621,40 @@ get_readable_date (const time_t file_time_raw)
 	} else if (file_date_age == 1) {
 		format = g_strdup(_("yesterday at %-I:%M %p"));
 	} else if (file_date_age < 7) {
-		format = g_strdup(_("%m/%-d/%y, %-I:%M %p"));
+		format = g_strdup(_("%A, %B %-d %Y at %-I:%M:%S %p"));
 	} else {
-		format = g_strdup(_("%m/%-d/%y, %-I:%M %p"));
+		format = g_strdup(_("%A, %B %-d %Y at %-I:%M:%S %p"));
 	}
 	
-	readable_date = strdup_strftime (format, file_time);
+	readable_date = gsearchtool_strdup_strftime (format, file_time);
 	g_free (format);
 	
 	return readable_date;
 }  
 
 gchar *
-strdup_strftime (const gchar *format, 
-		 struct tm *time_pieces)
+gsearchtool_strdup_strftime (const gchar *format, 
+                             struct tm *time_pieces)
 {
-	/* this function is based on eel */
+	/* This function is borrowed from eel's eel_strdup_strftime() */
 	GString *string;
 	const char *remainder, *percent;
-	char code[3], buffer[512];
+	char code[4], buffer[512];
 	char *piece, *result, *converted;
 	size_t string_length;
 	gboolean strip_leading_zeros, turn_leading_zeros_to_spaces;
+	char modifier;
+	int i;
 
+	/* Format could be translated, and contain UTF-8 chars,
+	 * so convert to locale encoding which strftime uses */
 	converted = g_locale_from_utf8 (format, -1, NULL, NULL, NULL);
 	g_return_val_if_fail (converted != NULL, NULL);
 	
 	string = g_string_new ("");
 	remainder = converted;
 
+	/* Walk from % character to % character. */
 	for (;;) {
 		percent = strchr (remainder, '%');
 		if (percent == NULL) {
@@ -364,6 +664,7 @@ strdup_strftime (const gchar *format,
 		g_string_append_len (string, remainder,
 				     percent - remainder);
 
+		/* Handle the "%" character. */
 		remainder = percent + 1;
 		switch (*remainder) {
 		case '-':
@@ -381,7 +682,7 @@ strdup_strftime (const gchar *format,
 			remainder++;
 			continue;
 		case '\0':
-			g_warning ("Trailing %% passed to strdup_strftime");
+			g_warning ("Trailing %% passed to gsearchtool_strdup_strftime");
 			g_string_append_c (string, '%');
 			continue;
 		default:
@@ -389,26 +690,53 @@ strdup_strftime (const gchar *format,
 			turn_leading_zeros_to_spaces = FALSE;
 			break;
 		}
+
+		modifier = 0;
+		if (strchr (SUS_EXTENDED_STRFTIME_MODIFIERS, *remainder) != NULL) {
+			modifier = *remainder;
+			remainder++;
+
+			if (*remainder == 0) {
+				g_warning ("Unfinished %%%c modifier passed to gsearchtool_strdup_strftime", modifier);
+				break;
+			}
+		} 
 		
 		if (strchr (C_STANDARD_STRFTIME_CHARACTERS, *remainder) == NULL) {
-			g_warning ("strdup_strftime does not support "
+			g_warning ("gsearchtool_strdup_strftime does not support "
 				   "non-standard escape code %%%c",
 				   *remainder);
 		}
 
-		code[0] = '%';
-		code[1] = *remainder;
-		code[2] = '\0';
+		/* Convert code to strftime format. We have a fixed
+		 * limit here that each code can expand to a maximum
+		 * of 512 bytes, which is probably OK. There's no
+		 * limit on the total size of the result string.
+		 */
+		i = 0;
+		code[i++] = '%';
+		if (modifier != 0) {
+#ifdef HAVE_STRFTIME_EXTENSION
+			code[i++] = modifier;
+#endif
+		}
+		code[i++] = *remainder;
+		code[i++] = '\0';
 		string_length = strftime (buffer, sizeof (buffer),
 					  code, time_pieces);
 		if (string_length == 0) {
+			/* We could put a warning here, but there's no
+			 * way to tell a successful conversion to
+			 * empty string from a failure.
+			 */
 			buffer[0] = '\0';
 		}
 
+		/* Strip leading zeros if requested. */
 		piece = buffer;
 		if (strip_leading_zeros || turn_leading_zeros_to_spaces) {
 			if (strchr (C_STANDARD_NUMERIC_STRFTIME_CHARACTERS, *remainder) == NULL) {
-				g_warning ("strdup_strftime does not support "
+				g_warning ("gsearchtool_strdup_strftime does not support "
 					   "modifier for non-numeric escape code %%%c%c",
 					   remainder[-1],
 					   *remainder);
@@ -427,53 +755,62 @@ strdup_strftime (const gchar *format,
 			}
 		}
 		remainder++;
+
+		/* Add this piece. */
 		g_string_append (string, piece);
 	}
 	
+	/* Convert the string back into utf-8. */
 	result = g_locale_to_utf8 (string->str, -1, NULL, NULL, NULL);
+
 	g_string_free (string, TRUE);
 	g_free (converted);
+
 	return result;
 }
 
-gchar *
-get_file_type_with_mime_type (const gchar *filename, 
-			      const gchar *mimetype)
+const char *
+get_file_type_for_mime_type (const gchar *filename, 
+                             const gchar *mimetype)
 {
+	const char *desc;
+	
 	if (filename == NULL || mimetype == NULL) {
-		return (gchar *)gnome_vfs_mime_get_description (GNOME_VFS_MIME_TYPE_UNKNOWN);
+		return gnome_vfs_mime_get_description (GNOME_VFS_MIME_TYPE_UNKNOWN);
 	}
 
+	desc = gnome_vfs_mime_get_description (mimetype);
+
 	if (g_file_test (filename, G_FILE_TEST_IS_SYMLINK)) {
-		if (g_ascii_strcasecmp (mimetype, GNOME_VFS_MIME_TYPE_UNKNOWN) == 0) {
-			GnomeVFSFileInfo *file_info;
+	
+		GnomeVFSFileInfo *file_info;
 		
-			file_info = gnome_vfs_file_info_new ();
-			gnome_vfs_get_file_info (filename, file_info, GNOME_VFS_FILE_INFO_DEFAULT);
+		file_info = gnome_vfs_file_info_new ();
+		gnome_vfs_get_file_info (filename, file_info, GNOME_VFS_FILE_INFO_DEFAULT);
 		
-			if (g_file_test (file_info->symlink_name, G_FILE_TEST_EXISTS) != TRUE) {
+		if (g_file_test (file_info->symlink_name, G_FILE_TEST_EXISTS) != TRUE) {
+		
+			if ((g_ascii_strcasecmp (mimetype, "x-special/socket") != 0) && 
+		            (g_ascii_strcasecmp (mimetype, "x-special/fifo") != 0)) {
+			    
 				gnome_vfs_file_info_unref (file_info);
 				return _("link (broken)");
 			}
-			
-			gnome_vfs_file_info_unref (file_info);
 		}
-		return g_strdup_printf (_("link to %s"), 
-			(gchar *)gnome_vfs_mime_get_description (mimetype));
+			
+		gnome_vfs_file_info_unref (file_info);
+		return g_strdup_printf (_("link to %s"), (desc != NULL) ? desc : mimetype);
 	}
-
-	return (gchar *)gnome_vfs_mime_get_description (mimetype);
+	return desc;
 } 
 
-gchar *
-get_file_icon_with_mime_type (const gchar *filename, 
-			      const gchar *mimetype) 
+GdkPixbuf *
+get_file_pixbuf_for_mime_type (const gchar *filename, 
+			       const gchar *mimetype) 
 {
-	const GnomeIconData *icon_data;
-	gchar *icon_name = NULL;	
-	gchar *icon_path = NULL;
-	int base_size;
-
+	GdkPixbuf *pixbuf;
+	gchar *icon_name = NULL;
+	
 	if (filename == NULL || mimetype == NULL) {
 		icon_name = g_strdup (ICON_THEME_REGULAR_ICON);
 	}
@@ -481,15 +818,33 @@ get_file_icon_with_mime_type (const gchar *filename,
 	         (g_ascii_strcasecmp (mimetype, "application/x-executable-binary") == 0)) {
 		icon_name = g_strdup (ICON_THEME_EXECUTABLE_ICON);
 	}
+	else if (g_ascii_strcasecmp (mimetype, "x-special/device-char") == 0) {
+		icon_name = g_strdup (ICON_THEME_CHAR_DEVICE);
+	}
+	else if (g_ascii_strcasecmp (mimetype, "x-special/device-block") == 0) {
+		icon_name = g_strdup (ICON_THEME_BLOCK_DEVICE);
+	}
+	else if (g_ascii_strcasecmp (mimetype, "x-special/socket") == 0) {
+		icon_name = g_strdup (ICON_THEME_SOCKET);
+	}
+	else if (g_ascii_strcasecmp (mimetype, "x-special/fifo") == 0) {
+		icon_name = g_strdup (ICON_THEME_FIFO);
+	}
 	else {
-		icon_name = gnome_icon_lookup (interface.icon_theme, NULL, filename, NULL, 
+		icon_name = gnome_icon_lookup (gtk_icon_theme_get_default (), NULL, filename, NULL, 
 					       NULL, mimetype, 0, NULL);
 	}
-
-	icon_path = gnome_icon_theme_lookup_icon (interface.icon_theme, icon_name, ICON_SIZE, 
-						  &icon_data, &base_size);
+	
+	pixbuf = (GdkPixbuf *) g_hash_table_lookup (search_command.pixbuf_hash, icon_name);
+	
+	if (pixbuf == NULL) {
+		pixbuf = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (), icon_name, 
+		                                   ICON_SIZE, 0, NULL);
+		g_hash_table_insert (search_command.pixbuf_hash, &icon_name, pixbuf);
+	}
+						  
 	g_free (icon_name);
-	return icon_path;	
+	return pixbuf;
 }
 
 gboolean
@@ -514,11 +869,9 @@ is_nautilus_running (void)
 gboolean
 is_component_action_type (const gchar *filename)
 {
-	gchar *mimeType = gnome_vfs_get_mime_type (filename);	
+	const char *mimeType = gnome_vfs_get_file_mime_type (filename, NULL, FALSE);	
 	GnomeVFSMimeActionType actionType = gnome_vfs_mime_get_default_action_type (mimeType);
 	
-	g_free (mimeType);
-	
 	if (actionType == GNOME_VFS_MIME_ACTION_TYPE_COMPONENT) {
 		return TRUE;
 	}
@@ -546,10 +899,8 @@ open_file_with_nautilus (const gchar *filename)
 gboolean
 open_file_with_application (const gchar *filename)
 {
-	gchar *mimeType = gnome_vfs_get_mime_type (filename);	
+	const char *mimeType = gnome_vfs_get_file_mime_type (filename, NULL, FALSE);	
 	GnomeVFSMimeApplication *mimeApp = gnome_vfs_mime_get_default_application (mimeType);
-	
-	g_free (mimeType);
 		
 	if (mimeApp) {
 		gint  argc;
@@ -577,7 +928,7 @@ open_file_with_application (const gchar *filename)
 gboolean
 launch_file (const gchar *filename)
 {
-	gchar *mime_type = gnome_vfs_get_mime_type (filename);	
+	const char *mime_type = gnome_vfs_get_file_mime_type (filename, NULL, FALSE);	
 	gboolean result = FALSE;
 	
 	if ((g_file_test (filename, G_FILE_TEST_IS_EXECUTABLE)) &&
@@ -585,7 +936,6 @@ launch_file (const gchar *filename)
 		result = g_spawn_command_line_async (filename, NULL);
 	}
 	
-	g_free (mime_type);
 	return result;
 }
 
@@ -625,7 +975,7 @@ gsearchtool_unique_filename (const gchar *dir,
 					    suffix);
 
 		g_free (retval);
-		retval = g_strconcat (dir, "/", filename, NULL);
+		retval = g_strconcat (dir, G_DIR_SEPARATOR_S, filename, NULL);
 		exists = g_file_test (retval, G_FILE_TEST_EXISTS);
 		g_free (filename);
 	}
