@@ -66,30 +66,31 @@ struct _FindOptionTemplate {
 	SearchConstraintType type;
 	gchar 	 *option;          /* the option string to pass to find or whatever */
 	gchar 	 *desc;            /* description */
+	gchar    *units;           /* units */
 	gboolean is_selected;     
 };
 	
 static FindOptionTemplate templates[] = {
-	{ SEARCH_CONSTRAINT_TEXT, "-exec grep -c '%s' {} \\;", N_("Contains the _text"), FALSE },
-	{ SEARCH_CONSTRAINT_SEPARATOR, NULL, NULL, TRUE },
-	{ SEARCH_CONSTRAINT_TIME_LESS, "-mtime -%d", N_("_Date modified less than (days)"), FALSE },
-	{ SEARCH_CONSTRAINT_TIME_MORE, "\\( -mtime +%d -o -mtime %d \\)", N_("Date modified more than (da_ys)"), FALSE },
-	{ SEARCH_CONSTRAINT_SEPARATOR, NULL, NULL, TRUE },
-	{ SEARCH_CONSTRAINT_NUMBER, "-size +%uc", N_("S_ize at least (kilobytes)"), FALSE }, 
-	{ SEARCH_CONSTRAINT_NUMBER, "-size -%uc", N_("Si_ze at most (kilobytes)"), FALSE },
-	{ SEARCH_CONSTRAINT_BOOL, "-size 0c \\( -type f -o -type d \\)", N_("File is empty"), FALSE },	
-	{ SEARCH_CONSTRAINT_SEPARATOR, NULL, NULL, TRUE },
-	{ SEARCH_CONSTRAINT_TEXT, "-user '%s'", N_("Owned by _user"), FALSE },
-	{ SEARCH_CONSTRAINT_TEXT, "-group '%s'", N_("Owned by _group"), FALSE },
-	{ SEARCH_CONSTRAINT_BOOL, "\\( -nouser -o -nogroup \\)", N_("Owner is unrecognized"), FALSE },
-	{ SEARCH_CONSTRAINT_SEPARATOR, NULL, NULL, TRUE },
-	{ SEARCH_CONSTRAINT_TEXT, "'!' -name '*%s*'", N_("Na_me does not contain"), FALSE },
-	{ SEARCH_CONSTRAINT_TEXT, "-regex '%s'", N_("Name matches regular e_xpression"), FALSE }, 
-	{ SEARCH_CONSTRAINT_SEPARATOR, NULL, NULL, TRUE },
-	{ SEARCH_CONSTRAINT_BOOL, "SHOW_HIDDEN_FILES", N_("Show hidden and backup files"), FALSE },
-	{ SEARCH_CONSTRAINT_BOOL, "-follow", N_("Follow symbolic links"), FALSE },
-	{ SEARCH_CONSTRAINT_BOOL, "INCLUDE_OTHER_FILESYSTEMS", N_("Include other filesystems"), FALSE },
-	{ SEARCH_CONSTRAINT_END, NULL, NULL, FALSE}
+	{ SEARCH_CONSTRAINT_TEXT, "'!' -type p -exec grep -c '%s' {} \\;", N_("Contains the _text"), NULL, FALSE },
+	{ SEARCH_CONSTRAINT_SEPARATOR, NULL, NULL, NULL, TRUE },
+	{ SEARCH_CONSTRAINT_TIME_LESS, "-mtime -%d", N_("_Date modified less than"), N_("days"), FALSE },
+	{ SEARCH_CONSTRAINT_TIME_MORE, "\\( -mtime +%d -o -mtime %d \\)", N_("Date modified more than"), N_("days"), FALSE },
+	{ SEARCH_CONSTRAINT_SEPARATOR, NULL, NULL, NULL, TRUE },
+	{ SEARCH_CONSTRAINT_NUMBER, "\\( -size %uc -o -size +%uc \\)", N_("S_ize at least"), N_("kilobytes"), FALSE }, 
+	{ SEARCH_CONSTRAINT_NUMBER, "\\( -size %uc -o -size -%uc \\)", N_("Si_ze at most"), N_("kilobytes"), FALSE },
+	{ SEARCH_CONSTRAINT_BOOL, "-size 0c \\( -type f -o -type d \\)", N_("File is empty"), NULL, FALSE },	
+	{ SEARCH_CONSTRAINT_SEPARATOR, NULL, NULL, NULL, TRUE },
+	{ SEARCH_CONSTRAINT_TEXT, "-user '%s'", N_("Owned by _user"), NULL, FALSE },
+	{ SEARCH_CONSTRAINT_TEXT, "-group '%s'", N_("Owned by _group"), NULL, FALSE },
+	{ SEARCH_CONSTRAINT_BOOL, "\\( -nouser -o -nogroup \\)", N_("Owner is unrecognized"), NULL, FALSE },
+	{ SEARCH_CONSTRAINT_SEPARATOR, NULL, NULL, NULL, TRUE },
+	{ SEARCH_CONSTRAINT_TEXT, "'!' -name '*%s*'", N_("Na_me does not contain"), NULL, FALSE },
+	{ SEARCH_CONSTRAINT_TEXT, "-regex '%s'", N_("Name matches regular e_xpression"), NULL, FALSE }, 
+	{ SEARCH_CONSTRAINT_SEPARATOR, NULL, NULL, NULL, TRUE },
+	{ SEARCH_CONSTRAINT_BOOL, "SHOW_HIDDEN_FILES", N_("Show hidden and backup files"), NULL, FALSE },
+	{ SEARCH_CONSTRAINT_BOOL, "-follow", N_("Follow symbolic links"), NULL, FALSE },
+	{ SEARCH_CONSTRAINT_BOOL, "INCLUDE_OTHER_FILESYSTEMS", N_("Include other filesystems"), NULL, FALSE },
+	{ SEARCH_CONSTRAINT_END, NULL, NULL, NULL, FALSE}
 }; 
 
 static GtkTargetEntry dnd_table[] = {
@@ -199,7 +200,7 @@ setup_case_insensitive_arguments (void)
 	g_spawn_command_line_sync ("grep -i 'string' /dev/null", NULL, &cmd_stderr, NULL, NULL);
 
 	if ((cmd_stderr != NULL) && (strlen (cmd_stderr) == 0)) {
-		templates[SEARCH_CONSTRAINT_CONTAINS_THE_TEXT].option = g_strdup ("-exec grep -i -c '%s' {} \\;");
+		templates[SEARCH_CONSTRAINT_CONTAINS_THE_TEXT].option = g_strdup ("'!' -type p -exec grep -i -c '%s' {} \\;");
 	}
 	g_free (cmd_stderr);
 
@@ -264,8 +265,37 @@ setup_find_name_options (gchar *filename)
 	return g_string_free (command, FALSE);
 }
 
+gboolean
+has_additional_constraints (void)
+{
+	GList *list;
+	
+	if (interface.selected_constraints != NULL) {
+	
+		for (list = interface.selected_constraints; list != NULL; list = g_list_next (list)) {
+			
+			SearchConstraint *constraint = list->data;
+							
+			switch (templates[constraint->constraint_id].type) {
+			case SEARCH_CONSTRAINT_BOOL:
+			case SEARCH_CONSTRAINT_NUMBER:
+			case SEARCH_CONSTRAINT_TIME_LESS:	
+			case SEARCH_CONSTRAINT_TIME_MORE:
+				return TRUE;
+			case SEARCH_CONSTRAINT_TEXT:
+				if (strlen (constraint->data.text) > 0) {
+					return TRUE;
+				} 
+			default:
+				break;
+			}
+		}
+	}
+	return FALSE;
+}
+
 gchar *
-build_search_command (void)
+build_search_command (gboolean first_pass)
 {
 	GString *command;
 	gchar *file_is_named_utf8;
@@ -277,9 +307,10 @@ build_search_command (void)
 
 	setup_case_insensitive_arguments ();
 	
-	file_is_named_utf8 = (gchar *) gtk_entry_get_text (GTK_ENTRY(gnome_entry_gtk_entry (GNOME_ENTRY(interface.file_is_named_entry))));
+	file_is_named_utf8 = g_strdup ((gchar *) gtk_entry_get_text (GTK_ENTRY(gnome_entry_gtk_entry (GNOME_ENTRY(interface.file_is_named_entry)))));
 
 	if (!file_is_named_utf8 || !*file_is_named_utf8) {
+		g_free (file_is_named_utf8);
 		file_is_named_utf8 = g_strdup ("*");
 	} 
 	else {
@@ -289,17 +320,19 @@ build_search_command (void)
 		gnome_entry_prepend_history (GNOME_ENTRY(interface.file_is_named_entry), TRUE, file_is_named_utf8);
 
 		if (strstr (locale, "*") == NULL) {
+			gchar *tmp;
+		
+			tmp = file_is_named_utf8;
 			file_is_named_utf8 = g_strconcat ("*", file_is_named_utf8, "*", NULL);
+			g_free (tmp);
 		}
 		
 		g_free (locale);
 	}
 	file_is_named_locale = g_locale_from_utf8 (file_is_named_utf8, -1, NULL, NULL, NULL);
 	
-	look_in_folder_utf8 = gnome_file_entry_get_full_path (GNOME_FILE_ENTRY(interface.look_in_folder_entry), FALSE);
+	look_in_folder_utf8 = gtk_file_chooser_get_current_folder (GTK_FILE_CHOOSER (interface.look_in_folder_entry));
 	look_in_folder_locale = g_locale_from_utf8 (look_in_folder_utf8, -1, NULL, NULL, NULL);
-	gnome_entry_prepend_history (GNOME_ENTRY(gnome_file_entry_gnome_entry (GNOME_FILE_ENTRY(interface.look_in_folder_entry))), 
-				     TRUE, look_in_folder_utf8);
 	g_free (look_in_folder_utf8);
 	
 	if (!file_extension_is (look_in_folder_locale, G_DIR_SEPARATOR_S)) {
@@ -313,51 +346,66 @@ build_search_command (void)
 	
 	command = g_string_new ("");
 	search_command.show_hidden_files = FALSE;
+	search_command.regex_string = NULL;
+	search_command.date_format_pref = NULL;
+	search_command.file_is_named_pattern = NULL;
+	
+	search_command.first_pass = first_pass;
+	if (search_command.first_pass == TRUE) {
+		search_command.quick_mode = FALSE;
+	}
 	
 	if ((GTK_WIDGET_VISIBLE(interface.additional_constraints) == FALSE) ||
-	    (interface.selected_constraints == NULL)) {
+	    (has_additional_constraints() == FALSE)) {
 	
-		gchar *locate;
-		gboolean disable_quick_search;
-		
-		locate = g_find_program_in_path ("locate");
-		file_is_named_backslashed = backslash_backslashes (file_is_named_locale);
+		file_is_named_backslashed = backslash_special_characters (file_is_named_locale);
 		file_is_named_escaped = escape_single_quotes (file_is_named_backslashed);
-
 		search_command.file_is_named_pattern = g_strdup(file_is_named_utf8);
-		disable_quick_search = gsearchtool_gconf_get_boolean ("/apps/gnome-search-tool/disable_quick_search");
 		
-		if ((disable_quick_search != TRUE) && (locate != NULL) 
-		    && (is_path_in_home_folder (look_in_folder_locale) != TRUE) 
-		    && (is_path_in_mount_folder (look_in_folder_locale) != TRUE)
-		    && (is_path_in_proc_folder (look_in_folder_locale) != TRUE) 
-		    && (is_path_in_dev_folder (look_in_folder_locale) != TRUE)
-		    && (is_path_in_var_folder (look_in_folder_locale) != TRUE)
-		    && (is_path_in_tmp_folder (look_in_folder_locale) != TRUE)) {	
-			g_string_append_printf (command, "%s %s '%s*%s'", 
-						locate,
-						locate_command_default_options,
-						look_in_folder_locale,
-						file_is_named_escaped);
-		} 
+		if (search_command.first_pass == TRUE) {
+		
+			gchar *locate;
+			gboolean disable_quick_search;
+		
+			locate = g_find_program_in_path ("locate");
+			disable_quick_search = gsearchtool_gconf_get_boolean ("/apps/gnome-search-tool/disable_quick_search");		
+			search_command.disable_second_pass = gsearchtool_gconf_get_boolean ("/apps/gnome-search-tool/disable_quick_search_second_pass");
+		
+			if ((disable_quick_search == FALSE) 
+			    && (locate != NULL) 
+			    && (is_quick_search_excluded_path (look_in_folder_locale) == FALSE)) {
+			    
+					g_string_append_printf (command, "%s %s '%s*%s'", 
+							locate,
+							locate_command_default_options,
+							look_in_folder_locale,
+							file_is_named_escaped);
+					search_command.quick_mode = TRUE;
+			}
+			else {		
+				g_string_append_printf (command, "find \"%s\" %s '%s' -xdev -print", 
+							look_in_folder_locale, 
+							find_command_default_name_option, 
+							file_is_named_escaped);
+			}
+			g_free (locate);
+		}
 		else {
-			g_string_append_printf (command, "find \"%s\" '!' -type p %s '%s' -xdev -print", 
+			g_string_append_printf (command, "find \"%s\" %s '%s' -xdev -print", 
 						look_in_folder_locale, 
 						find_command_default_name_option, 
 						file_is_named_escaped);
 		}
-		g_free (locate);
-	} 
+	}
 	else {
 		GList *list;
 		gboolean disable_mount_argument = FALSE;
 		
 		search_command.regex_matching_enabled = FALSE;
-		file_is_named_backslashed = backslash_backslashes (file_is_named_locale);
+		file_is_named_backslashed = backslash_special_characters (file_is_named_locale);
 		file_is_named_escaped = escape_single_quotes (file_is_named_backslashed);
-		search_command.file_is_named_pattern = g_strdup(file_is_named_utf8);
 		
-		g_string_append_printf (command, "find \"%s\" '!' -type p  %s", 
+		g_string_append_printf (command, "find \"%s\" %s", 
 					look_in_folder_locale,
 					setup_find_name_options (file_is_named_escaped));
 	
@@ -384,7 +432,7 @@ build_search_command (void)
 					gchar *escaped;
 					gchar *regex;
 					
-					escaped = backslash_backslashes (constraint->data.text);
+					escaped = backslash_special_characters (constraint->data.text);
 					regex = escape_single_quotes (escaped);
 					
 					if (regex != NULL) {	
@@ -400,7 +448,7 @@ build_search_command (void)
 					gchar *backslashed;
 					gchar *locale;
 					
-					backslashed = backslash_backslashes (constraint->data.text);
+					backslashed = backslash_special_characters (constraint->data.text);
 					escaped = escape_single_quotes (backslashed);
 					
 					locale = g_locale_from_utf8 (escaped, -1, NULL, NULL, NULL);
@@ -421,6 +469,7 @@ build_search_command (void)
 			case SEARCH_CONSTRAINT_NUMBER:
 				g_string_append_printf (command,
 					  		templates[constraint->constraint_id].option,
+							(constraint->data.number * 1024),
 					  		(constraint->data.number * 1024));
 				g_string_append_c (command, ' ');
 				break;
@@ -450,7 +499,9 @@ build_search_command (void)
 		g_string_append (command, "-print ");
 	}
 	g_free (file_is_named_locale);
+	g_free (file_is_named_utf8);
 	g_free (file_is_named_backslashed);
+	g_free (file_is_named_escaped);
 	g_free (look_in_folder_locale);
 	
 	return g_string_free (command, FALSE);
@@ -469,6 +520,12 @@ add_file_to_search_results (const gchar 	*file,
 	gchar *utf8_base_name, *utf8_dir_name;
 	gchar *base_name, *dir_name;
 	
+	if (g_hash_table_lookup_extended (search_command.file_hash, file, NULL, NULL) == TRUE) {
+		return;
+	}
+	
+	g_hash_table_insert (search_command.file_hash, g_strdup(file), NULL);
+	
 	mime_type = gnome_vfs_get_file_mime_type (file, NULL, FALSE);
 	 
 	description = get_file_type_for_mime_type (file, mime_type);
@@ -537,7 +594,7 @@ update_search_counts (void)
 {
 	gchar  *title_bar_string = NULL;
 	gchar  *message_string = NULL;
-	gchar  *stopped_string = g_strdup ("");
+	gchar  *stopped_string = NULL;
 	gchar  *tmp;
 	gint   total_files;
 
@@ -563,7 +620,7 @@ update_search_counts (void)
 						  total_files);
 	}
 
-	if (strlen (stopped_string) > 0) {
+	if (stopped_string != NULL) {
 		tmp = message_string;
 		message_string = g_strconcat (message_string, " ", stopped_string, NULL);
 		g_free (tmp);
@@ -585,6 +642,26 @@ update_search_counts (void)
 	g_free (stopped_string);
 }
 
+static void
+intermediate_file_count_update (void)
+{
+	gchar *string;
+	gint  count;
+	
+	count = gtk_tree_model_iter_n_children (GTK_TREE_MODEL(interface.model), NULL);
+	
+	if (count > 0) {
+		
+		string = g_strdup_printf (ngettext ("%d file found",
+		                                    "%d files found",
+		                                    count),
+		                          count);
+					  
+		gtk_label_set_text (GTK_LABEL (interface.results_label), string);
+		g_free (string);
+	}
+}
+
 static GtkWidget *
 make_list_of_templates (void)
 {
@@ -685,10 +762,14 @@ set_constraint_selected_state (gint 		constraint_id,
 			gtk_option_menu_set_history (GTK_OPTION_MENU(interface.constraint_menu), index);
 			interface.selected_constraint = (long)index;
 			gtk_widget_set_sensitive (interface.add_button, TRUE);
+			gtk_widget_set_sensitive (interface.constraint_menu, TRUE);
+			gtk_widget_set_sensitive (interface.constraint_menu_label, TRUE);
 			return;
 		}
 	}
 	gtk_widget_set_sensitive (interface.add_button, FALSE);
+	gtk_widget_set_sensitive (interface.constraint_menu, FALSE);
+	gtk_widget_set_sensitive (interface.constraint_menu_label, FALSE);
 }
 
 void
@@ -837,7 +918,7 @@ get_desktop_item_name (void)
 	g_string_append (gs, g_strdup_printf ("named=%s", file_is_named_locale));
 	g_free (file_is_named_locale);
 
-	look_in_folder_utf8 = gnome_file_entry_get_full_path (GNOME_FILE_ENTRY(interface.look_in_folder_entry), FALSE);
+	look_in_folder_utf8 = gtk_file_chooser_get_current_folder (GTK_FILE_CHOOSER (interface.look_in_folder_entry));
 	look_in_folder_locale = g_locale_from_utf8 (look_in_folder_utf8 != NULL ? look_in_folder_utf8 : "", 
 	                                            -1, NULL, NULL, NULL);
 	g_string_append (gs, g_strdup_printf ("&path=%s", look_in_folder_locale));
@@ -910,15 +991,18 @@ get_desktop_item_name (void)
 static void
 start_animation (void)
 {
-	gchar *title = NULL;
+	if (search_command.first_pass == TRUE) {
 	
-	title = g_strconcat (_("Searching..."), " - ", _("Search for Files"), NULL);
-	gtk_window_set_title (GTK_WINDOW (interface.main_window), title);
+		gchar *title = NULL;
 	
-	gtk_label_set_text (GTK_LABEL (interface.results_label), "");
-	gsearch_spinner_start (GSEARCH_SPINNER (interface.spinner));
+		title = g_strconcat (_("Searching..."), " - ", _("Search for Files"), NULL);
+		gtk_window_set_title (GTK_WINDOW (interface.main_window), title);
 	
-	g_free (title);
+		gtk_label_set_text (GTK_LABEL (interface.results_label), "");
+		gsearch_spinner_start (GSEARCH_SPINNER (interface.spinner));
+	
+		g_free (title);
+	}
 } 
 
 static void
@@ -961,8 +1045,8 @@ handle_popt_args (void)
 	}
 	if (PoptArgument.path != NULL) {
 		popt_args_found = TRUE;
-		gtk_entry_set_text (GTK_ENTRY(gnome_file_entry_gtk_entry (GNOME_FILE_ENTRY(interface.look_in_folder_entry))),
-				    g_locale_to_utf8 (PoptArgument.path, -1, NULL, NULL, NULL));
+		gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (interface.look_in_folder_entry),
+		                               g_locale_to_utf8 (PoptArgument.path, -1, NULL, NULL, NULL));
 	}	
 	if (PoptArgument.contains != NULL) {
 		popt_args_found = TRUE;	
@@ -1114,11 +1198,15 @@ handle_search_command_stdout_io (GIOChannel 	*ioc,
 					broken_pipe = TRUE;
 				}
 				else if (status == G_IO_STATUS_AGAIN) {
-					while (gtk_events_pending ()) {
-						if (search_data->running == MAKE_IT_QUIT) {
-							return FALSE;
-						}
-						gtk_main_iteration (); 				
+					if (gtk_events_pending ()) {
+						intermediate_file_count_update ();
+						while (gtk_events_pending ()) {
+							if (search_data->running == MAKE_IT_QUIT) {
+								return FALSE;
+							}
+							gtk_main_iteration (); 
+						}	
+ 				
 					}
 				}
 				
@@ -1187,11 +1275,14 @@ handle_search_command_stdout_io (GIOChannel 	*ioc,
 			g_timer_elapsed (timer, &duration);
 
 			if (duration > GNOME_SEARCH_TOOL_REFRESH_DURATION) {
-				while (gtk_events_pending ()) {
-					if (search_data->running == MAKE_IT_QUIT) {
-						return FALSE;
-					}
-					gtk_main_iteration ();		
+				if (gtk_events_pending ()) {
+					intermediate_file_count_update ();
+					while (gtk_events_pending ()) {
+						if (search_data->running == MAKE_IT_QUIT) {
+							return FALSE;
+						}
+						gtk_main_iteration ();		
+					}		
 				}
 				g_timer_reset (timer);
 			}
@@ -1203,29 +1294,51 @@ handle_search_command_stdout_io (GIOChannel 	*ioc,
 	}
 
 	if (condition != G_IO_IN || broken_pipe == TRUE) { 
+
+		g_io_channel_shutdown (ioc, TRUE, NULL);
 		
 		if (search_data->running == MAKE_IT_STOP) {
 			search_data->aborted = TRUE;
 		}
 		
-		search_data->lock = FALSE;
-		search_data->running = NOT_RUNNING;
-		search_data->not_running_timeout = TRUE;
-		g_hash_table_destroy (search_data->pixbuf_hash);
-		g_timeout_add (500, not_running_timeout_cb, NULL);
+		if ((search_data->aborted == FALSE)
+		     && (search_data->quick_mode == TRUE) 
+		     && (search_data->first_pass == TRUE) 
+		     && (search_data->disable_second_pass == FALSE)
+		     && (is_second_scan_excluded_path (search_data->look_in_folder) == FALSE)) { 
+		    
+			gchar *command;
+			
+			command = build_search_command (FALSE);
+			spawn_search_command (command);
+			
+			g_free (command);
+		}
+		else {
+			search_data->lock = FALSE;
+			search_data->running = NOT_RUNNING;
+			search_data->not_running_timeout = TRUE;
+			g_hash_table_destroy (search_data->pixbuf_hash);
+			g_hash_table_destroy (search_data->file_hash);
+			g_timeout_add (500, not_running_timeout_cb, NULL); 
 
-		update_search_counts ();
-		stop_animation ();
+			update_search_counts (); 
+			stop_animation ();
 
-		gtk_window_set_default (GTK_WINDOW(interface.main_window), interface.find_button);
-		gtk_widget_set_sensitive (interface.additional_constraints, TRUE);
-		gtk_widget_set_sensitive (interface.expander, TRUE);
-		gtk_widget_set_sensitive (interface.table, TRUE);
-		gtk_widget_set_sensitive (interface.find_button, TRUE);
-		gtk_widget_hide (interface.stop_button);
-		gtk_widget_show (interface.find_button);
+			gtk_window_set_default (GTK_WINDOW(interface.main_window), interface.find_button);
+			gtk_widget_set_sensitive (interface.additional_constraints, TRUE);
+			gtk_widget_set_sensitive (interface.expander, TRUE);
+			gtk_widget_set_sensitive (interface.table, TRUE);
+			gtk_widget_set_sensitive (interface.find_button, TRUE);
+			gtk_widget_hide (interface.stop_button);
+			gtk_widget_show (interface.find_button); 
 
-		g_io_channel_shutdown (ioc, TRUE, NULL);
+			/* Free the gchar fields of search_command structure. */
+			g_free (search_data->look_in_folder);
+			g_free (search_data->file_is_named_pattern);
+			g_free (search_data->regex_string);
+			g_free (search_data->date_format_pref);
+		}
 		return FALSE;
 	}
 	return TRUE;
@@ -1265,11 +1378,15 @@ handle_search_command_stderr_io (GIOChannel 	*ioc,
 					broken_pipe = TRUE;
 				}
 				else if (status == G_IO_STATUS_AGAIN) {
-					while (gtk_events_pending ()) {
-						if (search_data->running == MAKE_IT_QUIT) {
-							break;
+					if (gtk_events_pending ()) {
+						intermediate_file_count_update ();
+						while (gtk_events_pending ()) {
+							if (search_data->running == MAKE_IT_QUIT) {
+								break;
+							}
+							gtk_main_iteration ();
+
 						}
-						gtk_main_iteration ();
 					}
 				}
 				
@@ -1312,28 +1429,59 @@ handle_search_command_stderr_io (GIOChannel 	*ioc,
 				
 				GtkWidget *dialog;
 				
-				error_msgs = g_string_prepend (error_msgs, 
-				     _("While searching the following errors were reported.\n\n"));
-				
 				if (truncate_error_msgs) {
 					error_msgs = g_string_append (error_msgs, 
 				     		_("\n... Too many errors to display ..."));
 				}
 				
-				dialog = gsearch_alert_dialog_new (GTK_WINDOW (interface.main_window),
-		                	                           GTK_DIALOG_DESTROY_WITH_PARENT,
-						                   GTK_MESSAGE_ERROR,
-							           GTK_BUTTONS_OK,
-							           _("Error while searching for files."),
-							           error_msgs->str,
-							           NULL);
+				if (search_command.quick_mode != TRUE) {
+				     
+					dialog = gsearch_alert_dialog_new (GTK_WINDOW (interface.main_window),
+		                		                           GTK_DIALOG_DESTROY_WITH_PARENT,
+							                   GTK_MESSAGE_ERROR,
+								           GTK_BUTTONS_OK,
+								           _("The search results may be invalid."
+									     "  There were errors while performing this search."),
+								           NULL,
+								           NULL);			
 				
-				g_signal_connect (G_OBJECT (dialog),
-					"response",
-					G_CALLBACK (gtk_widget_destroy), NULL);
+					gsearch_alert_dialog_set_secondary_label (GSEARCH_ALERT_DIALOG (dialog), NULL);							   
+					gsearch_alert_dialog_set_details_label (GSEARCH_ALERT_DIALOG (dialog), error_msgs->str);
+					
+					g_signal_connect (G_OBJECT (dialog),
+						"response",
+						G_CALLBACK (gtk_widget_destroy), NULL);
+					
+					gtk_widget_show (dialog);
+				}
+				else {
+					GtkWidget *button;
+								       			       
+					dialog = gsearch_alert_dialog_new (GTK_WINDOW (interface.main_window),
+		                		                           GTK_DIALOG_DESTROY_WITH_PARENT,
+							                   GTK_MESSAGE_ERROR,
+								           GTK_BUTTONS_CANCEL,
+								           _("The search results may be invalid."
+									     "  Do you want to disable the quick search feature?"),
+								           NULL,
+								           NULL);
+									   
+					gsearch_alert_dialog_set_secondary_label (GSEARCH_ALERT_DIALOG (dialog), NULL);							   
+					gsearch_alert_dialog_set_details_label (GSEARCH_ALERT_DIALOG (dialog), error_msgs->str);
+					
+					button = gsearchtool_button_new_with_stock_icon (_("Disable _Quick Search"), GTK_STOCK_OK);
+					GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
+					gtk_widget_show (button);
+
+					gtk_dialog_add_action_widget (GTK_DIALOG (dialog), button, GTK_RESPONSE_OK);
+					gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
 				
-				gtk_widget_show (dialog);
-	
+					g_signal_connect (G_OBJECT (dialog),
+						"response",
+						G_CALLBACK (disable_quick_search_cb), NULL);
+					
+					gtk_widget_show (dialog);
+				}
 			}
 			truncate_error_msgs = FALSE;
 			g_string_truncate (error_msgs, 0);
@@ -1373,6 +1521,12 @@ spawn_search_command (gchar *command)
 		gtk_widget_destroy (dialog);
 		g_error_free (error);
 		g_strfreev (argv);
+		
+		/* Free the gchar fields of search_command structure. */
+		g_free (search_command.look_in_folder);
+		g_free (search_command.file_is_named_pattern);
+		g_free (search_command.regex_string);
+		g_free (search_command.date_format_pref);
 		return;
 	}	
 
@@ -1395,26 +1549,39 @@ spawn_search_command (gchar *command)
 		gtk_dialog_run (GTK_DIALOG (dialog));
 		gtk_widget_destroy (dialog);
 		g_error_free (error);
+		g_strfreev (argv);
+		
+		/* Free the gchar fields of search_command structure. */
+		g_free (search_command.look_in_folder);
+		g_free (search_command.file_is_named_pattern);
+		g_free (search_command.regex_string);
+		g_free (search_command.date_format_pref);
 		return;
 	}
 	
-	search_command.lock = TRUE;
-	search_command.aborted = FALSE;
-	search_command.running = RUNNING; 
-	search_command.pixbuf_hash = g_hash_table_new (g_str_hash, g_str_equal);
+	if (search_command.first_pass == TRUE) {
+		search_command.lock = TRUE;
+		search_command.aborted = FALSE;
+		search_command.running = RUNNING; 
+		search_command.pixbuf_hash = g_hash_table_new (g_str_hash, g_str_equal);
+		search_command.file_hash = g_hash_table_new (g_str_hash, g_str_equal);
+	
+		/* Get value of nautilus date_format key */
+		search_command.date_format_pref = gsearchtool_gconf_get_string ("/apps/nautilus/preferences/date_format");
 
-	gtk_window_set_default (GTK_WINDOW(interface.main_window), interface.stop_button);
-	gtk_widget_show (interface.stop_button);
-	gtk_widget_set_sensitive (interface.stop_button, TRUE);
-	gtk_widget_hide (interface.find_button);
-	gtk_widget_set_sensitive (interface.find_button, FALSE);
-	gtk_widget_set_sensitive (interface.results, TRUE);
-	gtk_widget_set_sensitive (interface.additional_constraints, FALSE);
-	gtk_widget_set_sensitive (interface.expander, FALSE);
-	gtk_widget_set_sensitive (interface.table, FALSE);
+		gtk_window_set_default (GTK_WINDOW(interface.main_window), interface.stop_button);
+		gtk_widget_show (interface.stop_button);
+		gtk_widget_set_sensitive (interface.stop_button, TRUE);
+		gtk_widget_hide (interface.find_button);
+		gtk_widget_set_sensitive (interface.find_button, FALSE);
+		gtk_widget_set_sensitive (interface.results, TRUE);
+		gtk_widget_set_sensitive (interface.additional_constraints, FALSE);
+		gtk_widget_set_sensitive (interface.expander, FALSE);
+		gtk_widget_set_sensitive (interface.table, FALSE);
 
-	gtk_tree_view_scroll_to_point (GTK_TREE_VIEW(interface.tree), 0, 0);
-	gtk_list_store_clear (GTK_LIST_STORE(interface.model));
+		gtk_tree_view_scroll_to_point (GTK_TREE_VIEW(interface.tree), 0, 0);	
+		gtk_list_store_clear (GTK_LIST_STORE(interface.model));
+	}
 	
 	ioc_stdout = g_io_channel_unix_new (child_stdout);
 	ioc_stderr = g_io_channel_unix_new (child_stderr);
@@ -1432,6 +1599,7 @@ spawn_search_command (gchar *command)
 
 	g_io_channel_unref (ioc_stdout);
 	g_io_channel_unref (ioc_stderr);
+	g_strfreev (argv);
 }
   
 static GtkWidget *
@@ -1440,6 +1608,7 @@ create_constraint_box (SearchConstraint *opt, gchar *value)
 	GtkWidget *hbox;
 	GtkWidget *label;
 	GtkWidget *entry;
+	GtkWidget *entry_hbox;
 	GtkWidget *button;
 	
 	hbox = gtk_hbox_new (FALSE, 12);
@@ -1464,13 +1633,15 @@ create_constraint_box (SearchConstraint *opt, gchar *value)
 			label = gtk_label_new_with_mnemonic (desc);
 			g_free (desc);
 		}
+		
+		/* add description label */
 		gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
 		
 		if (templates[opt->constraint_id].type == SEARCH_CONSTRAINT_TEXT) {
 			entry = gtk_entry_new();
 			if (value != NULL) {
 				gtk_entry_set_text (GTK_ENTRY(entry), value);
-				opt->data.text = g_strdup (value);
+				opt->data.text = value;
 			}
 		}
 		else {
@@ -1504,8 +1675,17 @@ create_constraint_box (SearchConstraint *opt, gchar *value)
 				  G_CALLBACK (constraint_activate_cb),
 				  NULL);
 				  
-		/* add label and text field */
-		gtk_box_pack_start(GTK_BOX(hbox), entry, TRUE, TRUE, 0);
+		/* add text field */
+		entry_hbox = gtk_hbox_new (FALSE, 6);
+		gtk_box_pack_start(GTK_BOX(hbox), entry_hbox, TRUE, TRUE, 0);
+		gtk_box_pack_start(GTK_BOX(entry_hbox), entry, TRUE, TRUE, 0);
+		
+		/* add units label */
+		if (templates[opt->constraint_id].units != NULL)
+		{
+			label = gtk_label_new_with_mnemonic (_(templates[opt->constraint_id].units));
+			gtk_box_pack_start (GTK_BOX(entry_hbox), label, FALSE, FALSE, 0);
+		}
 		
 		break;
 	default:
@@ -1572,7 +1752,6 @@ static GtkWidget *
 create_additional_constraint_section (void)
 {
 	GtkWidget *hbox;
-	GtkWidget *label;
 	gchar *desc;
 
 	interface.constraint = gtk_vbox_new (FALSE, 6);	
@@ -1581,13 +1760,13 @@ create_additional_constraint_section (void)
 	gtk_box_pack_end (GTK_BOX(interface.constraint), hbox, FALSE, FALSE, 0);
 
 	desc = g_strconcat (LEFT_LABEL_SPACING, _("A_vailable options:"), NULL);
-	label = gtk_label_new_with_mnemonic (desc);
+	interface.constraint_menu_label = gtk_label_new_with_mnemonic (desc);
 	g_free (desc);
 
-	gtk_box_pack_start (GTK_BOX(hbox), label, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX(hbox), interface.constraint_menu_label, FALSE, FALSE, 0);
 	
 	interface.constraint_menu = gtk_option_menu_new ();
-	gtk_label_set_mnemonic_widget (GTK_LABEL(label), GTK_WIDGET(interface.constraint_menu));
+	gtk_label_set_mnemonic_widget (GTK_LABEL(interface.constraint_menu_label), GTK_WIDGET(interface.constraint_menu));
 	gtk_option_menu_set_menu (GTK_OPTION_MENU(interface.constraint_menu), make_list_of_templates());
 	gtk_box_pack_start (GTK_BOX(hbox), interface.constraint_menu, TRUE, TRUE, 0);
 
@@ -1634,7 +1813,7 @@ create_search_results_section (void)
 	gtk_box_pack_start (GTK_BOX (hbox), interface.results_label, FALSE, FALSE, 0);
 	
 	window = gtk_scrolled_window_new (NULL, NULL);
-	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW(window), GTK_SHADOW_NONE);	
+	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW(window), GTK_SHADOW_IN);
 	gtk_container_set_border_width (GTK_CONTAINER(window), 0);
 	gtk_widget_set_size_request (window, 530, 160); 
 	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW(window),
@@ -1777,7 +1956,6 @@ create_main_window (void)
 	GtkWidget	*vbox;
 	GtkWidget 	*entry;
 	GtkWidget	*label;
-	GtkWidget 	*folder_entry;
 	GtkWidget 	*button;
 	GtkWidget 	*window;
 
@@ -1829,8 +2007,8 @@ create_main_window (void)
 	if (GTK_IS_ACCESSIBLE (gtk_widget_get_accessible(interface.file_is_named_entry)))
 	{
 		interface.is_gail_loaded = TRUE;
-		add_atk_namedesc (interface.file_is_named_entry, NULL, _("Enter the file name you want to search"));
-		add_atk_namedesc (entry, _("Name Contains Entry"), _("Enter the file name you want to search"));
+		add_atk_namedesc (interface.file_is_named_entry, NULL, _("Enter the file name you want to search."));
+		add_atk_namedesc (entry, _("Name Contains Entry"), _("Enter the file name you want to search."));
 	}	
      
 	g_signal_connect (G_OBJECT (interface.file_is_named_entry), "activate",
@@ -1847,33 +2025,24 @@ create_main_window (void)
 	
 	gtk_table_attach(GTK_TABLE(interface.table), label, 0, 1, 1, 2, GTK_FILL, 0, 0, 0);
 	
-	interface.look_in_folder_entry = gnome_file_entry_new ("gsearchtool-folder-entry", _("Browse"));
-	gnome_file_entry_set_directory_entry (GNOME_FILE_ENTRY (interface.look_in_folder_entry), TRUE);
-	g_object_set (G_OBJECT (interface.look_in_folder_entry), "use-filechooser", TRUE, NULL);
-	
-	entry = gnome_file_entry_gtk_entry (GNOME_FILE_ENTRY(interface.look_in_folder_entry));
-	gtk_label_set_mnemonic_widget (GTK_LABEL(label), interface.look_in_folder_entry);
-	folder_entry = gnome_file_entry_gnome_entry (GNOME_FILE_ENTRY(interface.look_in_folder_entry));
+	interface.look_in_folder_entry = gtk_file_chooser_button_new (_("Browse"));
+	gtk_file_chooser_set_action (GTK_FILE_CHOOSER (interface.look_in_folder_entry), GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER);
+	gtk_label_set_mnemonic_widget (GTK_LABEL(label), GTK_WIDGET (interface.look_in_folder_entry));		
 	gtk_table_attach (GTK_TABLE(interface.table), interface.look_in_folder_entry, 1, 2, 1, 2, GTK_EXPAND | GTK_FILL | GTK_SHRINK, 0, 0, 0);
 	
 	if (interface.is_gail_loaded)
 	{ 
-		add_atk_namedesc (GTK_WIDGET(folder_entry), NULL, _("Enter the folder name where you want to start the search"));
-		add_atk_namedesc (GTK_WIDGET(entry), _("Look in Folder Entry"), _("Enter the folder name where you want to start the search"));
+		add_atk_namedesc (GTK_WIDGET(interface.look_in_folder_entry), NULL, _("Select the folder where you want to start the search."));
 	}
 	
-	g_signal_connect (G_OBJECT (entry), "key_press_event",
-			  G_CALLBACK (look_in_folder_key_press_cb),
-			  NULL);
-	
-	g_signal_connect (G_OBJECT (interface.look_in_folder_entry), "changed",
+	g_signal_connect (G_OBJECT (interface.look_in_folder_entry), "current-folder-changed",
 			  G_CALLBACK (constraint_entry_changed_cb),
 			  NULL);
 			  
 	locale_string = g_get_current_dir ();
 	utf8_string = g_filename_to_utf8 (locale_string, -1, NULL, NULL, NULL);
 	
-	gtk_entry_set_text (GTK_ENTRY(entry), utf8_string);
+	gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (interface.look_in_folder_entry), utf8_string);
 	g_free (locale_string);
 	g_free (utf8_string);
 	
@@ -2005,7 +2174,7 @@ set_clone_command (gint *argcp, gchar ***argvp, gpointer client_data, gboolean e
 	g_free (tmp);
 	g_free (file_is_named_locale);
 
-	look_in_folder_utf8 = gnome_file_entry_get_full_path (GNOME_FILE_ENTRY(interface.look_in_folder_entry), FALSE);
+	look_in_folder_utf8 = gtk_file_chooser_get_current_folder (GTK_FILE_CHOOSER (interface.look_in_folder_entry));
 	look_in_folder_locale = g_locale_from_utf8 (look_in_folder_utf8 != NULL ? look_in_folder_utf8 : "", 
 	                                            -1, NULL, NULL, NULL);
 	g_free (look_in_folder_utf8);
