@@ -78,13 +78,18 @@ extern struct _SearchStruct {
 	gchar			*look_in_folder;
 	gchar           	*file_is_named_pattern;
 	gchar 	  		*regex_string;
+	gchar                   *date_format_pref;
 	gboolean		lock;	
 	gboolean		show_hidden_files;
 	gboolean		regex_matching_enabled;
 	gboolean		not_running_timeout;
 	gboolean		aborted;
+	gboolean		quick_mode;
+	gboolean		first_pass;
+	gboolean		disable_second_pass;
 	RunStatus        	running;
 	GHashTable              *pixbuf_hash;
+	GHashTable		*file_hash;
 } search_command;
 
 extern struct _InterfaceStruct {
@@ -102,6 +107,7 @@ extern struct _InterfaceStruct {
 	GtkWidget		*add_button;
 	GtkWidget 		*additional_constraints;
 	GtkWidget		*constraint_menu;
+	GtkWidget		*constraint_menu_label;
 	GtkWidget 		*constraint;
 	GtkWidget		*results_label;
 	GtkWidget       	*results;
@@ -128,7 +134,7 @@ struct _SearchConstraint {
 };
 
 gchar *
-build_search_command 		(void);
+build_search_command 		(gboolean first_pass);
 
 void
 spawn_search_command 		(gchar *command);
